# -*- coding: utf-8 -*-
"""
cini_selector: Module that work with the Spanish CINI code notification. Helping to verify if the code are valid or not. And help on each level The diffewrent options that the user have.
"""
__author__  = "Robert Rijnbeek"
__email__   = "robert270384@gmail.com"
__version__ = "0.0.5"

# ======== IMPORTS ===========

import json

from os.path import join, dirname, abspath


class CINI_Constructor():
    def __init__(self):
        self.cini_dict = {}
        self.ImportConfigurationFile()
        self.cini_1 = ""
        self.cini_2 = ""
        self.cini_3 = ""
        self.cini_4 = ""
        self.cini_5 = ""
        self.cini_6 = ""
        self.cini_7 = ""
        self.cinicode = ""
        self.selector_list = []

# ========= INIT FUNCTION ==============
    def ImportConfigurationFile(self): 
        try:
            file1 = open(join(dirname(abspath(__file__)),"CONFIG_FILE","CINI_Configuration_File.json"),"r")
            CINI_STRING = file1.read()
            CINI_DICT =json.loads(CINI_STRING)
            self.cini_dict = CINI_DICT
        except Exception as exc:
            assert print(f"ERROR: Importing configuration file:{exc}")

# =========== GETTERS ================
    def CINI_getter(self):
        return self.cinicode

    def Selector_CINI_getter(self):
        return self.selector_list
    
# ============SETTER =================
    def CINI_Builder(self):
        try:
            self.cinicode = "".join(['I',self.cini_1,self.cini_2,self.cini_3,self.cini_4,self.cini_5,self.cini_6,self.cini_7])
            return True
        except Exception as exc:
            print(f"ERROR: Unespected error building the CINI code:{exc}")
            return False

    def CINI_reset(self):
        try:
            self.cini_1 = ""
            self.cini_2 = ""
            self.cini_3 = ""
            self.cini_4 = ""
            self.cini_5 = ""
            self.cini_6 = ""
            self.cini_7 = ""
            self.cinicode = ""
            return True
        except Exception as exc:
            print(f"ERROR: Unespected error in method 'CINI_reset' :{exc}")
            return False
    
    def CINI_Setter_by_Level(self,VALUE,LEVEL):
        try:
            if LEVEL in range(1,8):
                value = str(VALUE)
                if LEVEL == 1:
                    self.cini_1 = value
                elif LEVEL == 2:
                    self.cini_2 = value
                elif LEVEL == 3:
                    self.cini_3 = value
                elif LEVEL == 4:
                    self.cini_4 = value
                elif LEVEL == 5:
                    self.cini_5 = value
                elif LEVEL == 6:
                    self.cini_6 = value
                elif LEVEL == 7:
                    self.cini_7 = value
                return True
            else:
                print("ERROR: defining the CINI code")
                return False
        except Exception as exc:
            print(f"ERROR: Unespected error:{exc}")
            return False

    def Define_Selector(self,SELECTOR_LIST):
        try:
            if len(SELECTOR_LIST)>7: 
                print(f"ERROR: The argument '{SELECTOR_LIST}' has more than 7 arguments")
                return False
            selectors = SELECTOR_LIST
            cini_array = self.cini_dict["cini"]
            level = 0
            while len(selectors) >0 :
                level += 1 
                filter_code = selectors[0]
                find_value = False
                for ind,row in enumerate(cini_array):
                    cod = row["cod"]
                    if str(cod) == str(filter_code):
                        if level == 7:
                            self.CINI_Setter_by_Level(filter_code,level)
                            cini_array = []
                            selectors.pop(0)
                            find_value = True
                            break
                        else:
                            cini_array = cini_array[ind]["next"]
                            selectors.pop(0)
                            self.CINI_Setter_by_Level(filter_code,level)
                            find_value = True
                            break
                if find_value is False:
                    print(f"ERROR: code value '{filter_code}' at level '{level}' does not exist")
                    return False
            
            selector_list =[]
            for row in cini_array:
                cod = row["cod"]
                des = row["des"]
                selector_list.append([cod,des])
            
            self.selector_list = selector_list
            return True
        except Exception as exc:
            print(f"ERROR: Unespected error in 'Define_Selector':{exc}")
            return False

# ================= VERIFICATION ===================
    
    def Is_CINI_Complete(self):
        try:
            if "" in (self.cini_1,self.cini_2,self.cini_3,self.cini_4,self.cini_5,self.cini_6,self.cini_7):
                return False
            else:
                return True
        except Exception as exc:
            print(f"ERROR: Unespected error in the method: 'Is_CINI_Complete': {exc}")
            return False

if __name__ == "__main__":

    pass


from setuptools import setup, find_packages

import re, io, os

class getPackageInfo:
    def __init__(self):
        self._package_module_name = None
        self._package_main_file = None
        self._git_config_file = None
        self._long_description = None
        self._description = None
        self._version = None
        self._author = None
        self._author_email = None
        self._git_host = None
        self._package_name = None
        self._git_issues_host = None
        self._required_packages = None
        self.__Process()
    
    def __Process(self):
        self.__getPackageModuleNameFromProject()
        self.__getPacketInitFileContent()
        self.__getconfigGIT()
        self.__getLongDescription()
        self.__getDescription()
        self.__getVersion()
        self.__getAuthor()
        self.__getAuthorEmail()
        self.__getGITHost()
        self.__getPackageName()
        self.__getGITIssuesHost()
        self.__getRequiredPackageList()

    def __getRequiredPackageList(self):
        dependencies =[]
        with open("requirements.txt", "r", encoding="utf-8") as fh:
            lines = fh.readlines()
            for line in lines:
                cleanline = re.sub('\s+', '', line)
                if (not cleanline == ""):
                    dependencies.append(re.sub('\s+', '', line))
        self._required_packages = dependencies
        return True

    def __getPackageModuleNameFromProject(self):
        for dir in next(os.walk(".\src"))[1]:
            if not ".egg-info" in str(dir):
                self._package_module_name = dir
                return True

    def __getLongDescription(self):
        with open("PIPY_README.md", "r", encoding="utf-8") as fh:
            self._long_description = fh.read()
            return  True

    def __getPacketInitFileContent(self):
        self._package_main_file = io.open(
                os.path.join('src', self._package_module_name,'__init__.py'),
                encoding='utf_8_sig'
            ).read()
        return True

    def __getconfigGIT(self):
        with open(os.path.join(os.getcwd(), '.git', 'config'), "r", encoding="utf-8") as fh:
            self._git_config_file = fh.read()
            return  True

    def __getDescription(self):
        self._description = re.search(
                r'(?<=\"\"\")((.|\n)*)(?=\"\"\")', 
                self._package_main_file
            ).group(1)
        return True

    def __getVersion(self):
        self._version = re.search(
            r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
            self._package_main_file
        ).group(1)
        return True

    def __getAuthor(self):
        self._author = re.search(
            r'__author__\s*=\s*[\'"]([^\'"]*)[\'"]',
            self._package_main_file
        ).group(1)
        return True
    
    def __getAuthorEmail(self):
        self._author_email = re.search(
                r'__email__\s*=\s*[\'"]([^\'"]*)[\'"]',
                self._package_main_file
            ).group(1)
        return True

    def __getGITHost(self):
        self._git_host = re.search(
                r'(?<=\[remote "origin"\]\n\turl = )((.|\n)*)(?=\.git)',
                self._git_config_file
            ).group(1)
        return True

    def __getGITIssuesHost(self):
        self._git_issues_host = self._git_host + "/issues"
        return True

    def __getPackageName(self):
        self._package_name = self._git_host.rsplit('/', 1)[-1]
        return True

    @property
    def package_name(self):
        return self._package_name

    @property
    def long_description(self):
        return self._long_description

    @property
    def description(self):
        return self._description

    @property
    def version(self):
        return self._version

    @property
    def author(self):
        return self._author

    @property
    def author_email(self):
        return self._author_email

    @property
    def git_host(self):
        return self._git_host

    @property
    def git_issues_host(self):
        return self._git_issues_host

    @property
    def required_packages(self):
        return self._required_packages


PACKAGE_INFO = getPackageInfo()

setup(
    name = PACKAGE_INFO.package_name,
    version = PACKAGE_INFO.version,
    author = PACKAGE_INFO.author ,
    author_email = PACKAGE_INFO.author_email ,
    description = PACKAGE_INFO.description,
    long_description = PACKAGE_INFO.long_description,
    long_description_content_type = "text/markdown",
    url = PACKAGE_INFO.git_host,
    project_urls = {
        "Bug Tracker": PACKAGE_INFO.git_issues_host,
    },
    data_files=[
        (
            '.git.',
            ['.git/config']
        ),
        (
            '.',
            ["requirements.txt"]
        )
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir = { "": "src"},
    packages = find_packages(where = "src"),
    python_requires = ">=3.6",
    install_requires=PACKAGE_INFO.required_packages,
    include_package_data = True
)